#include "itext.h"
#include "xtext.h"
#include "xjp.h"
#include <time.h>


#define BG_PIXEL	0x0000
#define FG_PIXEL	0xFFFF

static int _day_of_week(int,int,int);
static int _day_of_month(int,int);
static Pixmap _mask_cursor(Display *,Window,GC,Pixmap,int,int);
static Pixmap _cale_cursor(Display *,my_window_t *,GC *,int *,int *);
static Pixmap _help_cursor(Display *,my_window_t *,GC *,int *,int *,char *);
static Pixmap _time_cursor(Display *,my_window_t *,GC *,int *,int *);
static Pixmap _version_cursor(Display *,my_window_t *,GC *,int *,int *);

static int 
_day_of_week(int year,int mon,int day)
{
	int yo;
	if(mon < 3){ year--; mon += 12;}
	yo = (year + year/4 - year/100 + year/400 + (13*mon + 8)/5 + day) % 7;
	return yo;
}
static int 
_day_of_month(int year,int mon)
{
	if(mon == 2 || mon == 4 || mon == 6 || mon == 9 || mon == 11){
		if(mon == 2){
			if(year % 400 == 0 || ((year % 4 == 0) && (year % 100 != 0))){
				return 29;
			}else{
				return 28;
			}
		}else{
			return 30;
		}
	}else{
		return 31;
	}
}
static Pixmap 
_mask_cursor(Display *disp,Window win,GC gc,Pixmap pix,int w,int h)
{
	XImage *im,*im1;
	Pixmap mask;
	int x,y;

	if(gc == 0 || pix == 0) return 0;

	if((im = XGetImage(disp,pix,0,0,w,h,AllPlanes,XYPixmap)) == NULL) return 0;
	if((im1 = XGetImage(disp,pix,0,0,w,h,AllPlanes,XYPixmap)) == NULL) return 0;
	for(x=0;x<w;x++){
		for(y=0;y<h;y++){
			if(XGetPixel(im,x,y) == 0x1){
				XPutPixel(im1,x,y,0x1);
			}else{
				XPutPixel(im1,x,y,0x0);
			}
		}
	}
	mask = XCreatePixmap(disp,win,w,h,1);
	XPutImage(disp,mask,gc,im1,0,0,0,0,w,h);

	XDestroyImage(im);
	XDestroyImage(im1);

	return mask;
}
static Pixmap 
_cale_cursor(Display *disp,my_window_t *win,GC *gc,int *w,int *h)
{
	Pixmap pix;
	XRectangle ink,log;
	char *base = "25",str[12];
	time_t cur_time;
	struct tm *tmval;
	int week,day;
	int i,len,x,y,xx,xp,yp,xpad,ypad;

	XmbTextExtents(win->fs_s,base,strlen(base),&ink,&log);

	*w = log.width * 7 + 30;
	*h = log.height * 7 + 10;

	pix = XCreatePixmap(disp,win->win,*w,*h,1);
	if(*gc == 0) *gc = XCreateGC(disp,pix,0,0);

	XSetForeground(disp,*gc,BG_PIXEL);
	XFillRectangle(disp,pix,*gc,0,0,*w,*h);
	XSetForeground(disp,*gc,FG_PIXEL);

	cur_time = time(NULL);
	tmval = localtime(&cur_time);
	week = _day_of_week(tmval->tm_year + 1900,tmval->tm_mon + 1,1);
	day = _day_of_month(tmval->tm_year + 1900,tmval->tm_mon + 1);
	xp = *w / 7;
	yp = *h / 7;
	xpad = xp/2;
	ypad = 5;
	x = xpad + (week * xp);
	y = yp + ypad;
	for(i=1;i<=day;i++){
		sprintf(str,"%d",i);
		len = strlen(str);
		xx = x;
		if(len == 2){
			XmbTextExtents(win->fs_s,str,len,&ink,&log);
			xx = x - ink.width / 2;
		}
		XmbDrawString(disp,pix,win->fs_s,*gc,xx,y,str,len);

		if((x += xp) >= (xp * 7)){
			x = xpad;
			y += yp;
		}
	}
	strftime(str,sizeof(str),"%y/%m",tmval);
	XmbTextExtents(win->rubi_fs,str,strlen(str),&ink,&log);

	x = *w - ink.width - 3;
	y = *h - 2;
	XmbDrawString(disp,pix,win->rubi_fs,*gc,x,y,str,strlen(str));

	return pix;
}
static Pixmap 
_help_cursor(Display *disp,my_window_t *win,GC *gc,int *w,int *h,char *help)
{
	Pixmap pix;
	XRectangle ink,log;
	char *p,temp[128];
	int i=0,line=0,height=0,width=0;

	if(help == NULL) return 0;

	for(p=help;*p;p++){
		if(*p == '\n'){
			temp[i] = '\0';
			XmbTextExtents(win->fs_s,temp,i,&ink,&log);
			if(width < log.width) width = log.width;
			if(height < log.height) height = log.height;
			i=0;
			line++;

			continue;
		}
		if( *(p + 1) == '\0'){
			temp[i++] = *p;
			temp[i] = '\0';
			XmbTextExtents(win->fs_s,temp,i,&ink,&log);
			if(width < log.width) width = log.width;
			if(height < log.height) height = log.height;
			line++;

			break;
		}

		temp[i++] = *p;
	}
	*w = width + 10;
	*h = height * line + 10;
	pix = XCreatePixmap(disp,win->win,*w,*h,1);

	if(*gc == 0) *gc = XCreateGC(disp,pix,0,0);
	XSetForeground(disp,*gc,BG_PIXEL);
	XFillRectangle(disp,pix,*gc,0,0,*w,*h);
	XSetForeground(disp,*gc,FG_PIXEL);

	i=0;
	line = 0;
	for(p=help;*p;p++){
		if(*p == '\n'){
			temp[i] = '\0';
			XmbDrawString(disp,pix,win->fs_s,*gc,5,height*line+height+5,temp,i);
			i=0;
			line++;

			continue;
		}
		if(*(p + 1) == '\0'){
			temp[i++] = *p;
			temp[i] = '\0';
			XmbDrawString(disp,pix,win->fs_s,*gc,5,height*line+height+5,temp,i);
			line++;

			break;
		}

		temp[i++] = *p;
	}

	return pix;
}
static Pixmap 
_time_cursor(Display *disp,my_window_t *win,GC *gc,int *w,int *h)
{
	time_t nowTime;
	struct tm *tmval;
	Pixmap pix;
	XRectangle ink,log;
	char str[128];

	nowTime = time(NULL);
	tmval = localtime(&nowTime);
	strftime(str,sizeof(str),"%Y/%m/%d %H:%M:%S",tmval);

	XmbTextExtents(win->fs_s,str,strlen(str),&ink,&log);
	*w = log.width + 10;
	*h = log.height + 10;

	pix = XCreatePixmap(disp,win->win,*w,*h,1);

	if(*gc == 0) *gc = XCreateGC(disp,pix,0,0);

	XSetForeground(disp,*gc,BG_PIXEL);
	XFillRectangle(disp,pix,*gc,0,0,*w,*h);
	XSetForeground(disp,*gc,FG_PIXEL);

	XmbDrawString(disp,pix,win->fs_s,*gc,5,*h-5,str,strlen(str));

	return pix;
}
static Pixmap 
_version_cursor(Display *disp,my_window_t *win,GC *gc,int *w,int *h)
{
	Pixmap pix;
	XRectangle ink,log;
	char str[4][128];
	int i,j,s,x,y;

	sprintf(str[0],"%s  Ver. %s",XTEXT_NAME,XTEXT_VERSION);
	sprintf(str[1],": %s",XTEXT_CREATER);
	sprintf(str[2],"E-Mail: %s",XTEXT_EMAIL);
	sprintf(str[3],"HP: %s",XTEXT_HOMEPAGE);


	s = sizeof(str) / 128;

	x = 0;
	j = 0;
	for(i=0;i<s;i++){
		int len;
		len = strlen(str[i]);
		if( len > j ){
		       	x = i;
			j = len;
		}
	}	

	XmbTextExtents(win->fs_s,str[x],strlen(str[x]),&ink,&log);
	*w = log.width + 10;
	*h = log.height * s + 20;

	pix = XCreatePixmap(disp,win->win,*w,*h,1);

	if(*gc == 0) *gc = XCreateGC(disp,pix,0,0);

	XSetForeground(disp,*gc,BG_PIXEL);
	XFillRectangle(disp,pix,*gc,0,0,*w,*h);
	XSetForeground(disp,*gc,FG_PIXEL);

	y = 0;
	for(i=0;i<s;i++){

		y += log.height + 2;
		XmbDrawString(disp,pix,win->fs_s,*gc,2,y,str[i],strlen(str[i]));
	}

	return pix;
}
int 
xtextCursorType(Display *disp,my_window_t *win,char *hstr,char *title)
{
	static Cursor cursor;
	static GC gc;
	static int type;
	static char title0[128];

	Pixmap pix=0,mask=0;
	int w,h;

	if(win->cursor == 2){
		if(title == NULL){
			title0[0] = '\0';
			return 0;
		}
		if(type == 2 && strcmp(title0,title) == 0) return 0;
	}else{
		if( type == win->cursor ) return 0;
	}

	if(type != 0 && win->cursor == 0){
		XUndefineCursor(disp,win->win);
	}

	if(title != NULL) strcpy(title0,title);
	type = win->cursor;

	switch(type){
		case 1:
			pix = (Pixmap)_help_cursor(disp,win,&gc,&w,&h,"");
		break;
		case 2:
			if(title != NULL)
				pix = (Pixmap)_help_cursor(disp,win,&gc,&w,&h,title);
		break;
		case 3:
			pix = (Pixmap)_version_cursor(disp,win,&gc,&w,&h);
		break;
		case 4:
			pix = (Pixmap)_help_cursor(disp,win,&gc,&w,&h,hstr);
		break;
		/*
		case 2:
			pix = (Pixmap)_time_cursor(disp,win,&gc,&w,&h);
		break;
		case 3:
			pix = (Pixmap)_cale_cursor(disp,win,&gc,&w,&h);
		break;
		case 4: 
			pix = (Pixmap)_help_cursor(disp,win,&gc,&w,&h,help);
		break;
		*/

		default:case 0:
			return 0;
		break;
	}

	if((mask = (Pixmap)_mask_cursor(disp,win->win,gc,pix,w,h)) > 0){ 
		XColor c0,c1;

		c0.pixel = win->pointer_fg_pixel;
		XQueryColor(disp,win->cmap,&c0);

		c1.pixel = win->rubi_bg_pixel;
		XQueryColor(disp,win->cmap,&c1);

		if(cursor) 
			XFreeCursor(disp,cursor);

		cursor = XCreatePixmapCursor(disp,(Pixmap)pix,(Pixmap)mask,&c0,&c1,10,h - 10);
		XDefineCursor(disp,win->win,cursor);
	}

	if(mask) XFreePixmap(disp,mask);
	if(pix) XFreePixmap(disp,pix);

	return 0;
}
Cursor xtextGetFontCursor(Display *disp,my_window_t *win,char *fcur)
{
	Cursor cursor;
	Pixmap pix = 0, mask = 0;
	XRectangle ink,log;
	XColor c0,c1;
	GC gc = NULL;
	int width,height,len;

	len = strlen(fcur);

	XmbTextExtents(win->rubi_fs,fcur,len,&ink,&log);
	width = log.width + 10;
	height = log.height + 20;

	if((pix = (Pixmap)XCreatePixmap(disp,win->win,width,height,1)) == 0){
		return 0;
	}

	gc = XCreateGC(disp,pix,0,0);
	XSetForeground(disp,gc,BG_PIXEL);
	XFillRectangle(disp,pix,gc,0,0,width,height);
	XSetForeground(disp,gc,FG_PIXEL);
	XmbDrawString(disp,pix,win->rubi_fs,gc,2,height-10,fcur,len);

	if((mask = (Pixmap)_mask_cursor(disp,win->win,gc,pix,width,height)) == 0){
		XFreePixmap(disp,pix);
		return 0; 
	}
	c0.pixel = c1.pixel = win->rubi_fg_pixel;

	XQueryColor(disp,win->cmap,&c0);
	XQueryColor(disp,win->cmap,&c1);

	cursor = XCreatePixmapCursor(disp,(Pixmap)pix,(Pixmap)mask,&c0,&c1,width/2,height/2);

	if(gc != NULL) XFreeGC(disp,gc);
	if(pix) XFreePixmap(disp,pix);
	if(mask) XFreePixmap(disp,mask);

	return cursor;
}

