#include "itext.h"
#include "xtext.h"
#include "xjp.h"
#include <dirent.h>
#include <pwd.h>

static int _offt_data_free(offset_data_t **head,offset_data_t **tail)
{
	offset_data_t *ot,*ott;

	for(ot = *head;ot != NULL;ot = ott){
		ott = ot->next;
		free(ot);
	}

	*head = NULL;
	*tail = NULL;

	return 0;
}
static int _offt_data_add(offset_data_t **head,offset_data_t **tail,offset_data_t *im)
{
        if(*tail){
                im->next = NULL;
                im->prev = *tail;

                (*tail)->next = im;
                *tail = im;
        }else{
                im->next = NULL;
                im->prev = NULL;

                *tail = im;
                *head = im;
        }

	return 0;
}
static offset_data_t *_offt_data_alloc()
{
	offset_data_t *ot;

	if((ot = (offset_data_t *)malloc(sizeof(offset_data_t))) == NULL)
			return NULL;

	ot->font[0] = '\0';
	ot->c1 = 0;
	ot->c2 = 0;
	ot->x_off = 0;
	ot->y_off = 0;
	ot->anti = 0;

	return ot;
}
static int _offt_data_insert(offset_data_t **head,offset_data_t **tail,char *font,unsigned int c1_s,unsigned int c2_s,int xoff,int yoff,int anti)
{
	offset_data_t *ot;

	if((ot = (offset_data_t *)_offt_data_alloc()) == NULL){
		fprintf(stderr,"Error: off_data_alloc() xtext_f15.c\n");
		return -1;
	}

	strcpy(ot->font,font);
	ot->x_off = xoff;
	ot->y_off = yoff;
	ot->anti = anti;

	ot->c1 = c1_s;
	ot->c2 = c2_s;

	_offt_data_add(head,tail,ot);

	return 0;
}
static int _parse_data(char *data,char *str,unsigned int *c0,unsigned int *c1,int *xoff,int *yoff)
{
	char *p,temp[8192];
	int i = 0,k = 0,f = 0;

	for(p=data;*p;p++){

		if(*p == '\"'){
			if( k == 0 ){
				k = 1; i = 0;
				continue;
			}else{
				temp[i] = '\0';

				if( f == 0 ){
					strcpy(str,temp);

				}else if( f == 1 ){
					if(temp[0] == '0' && temp[1] == 'x')
						*c0 = strtol(temp,NULL,0);
				}else if( f == 2 ){
					if(temp[0] == '0' && temp[1] == 'x')
						*c1 = strtol(temp,NULL,0);
				}else if( f == 3 ){
					*xoff = atoi(temp);
				}else if( f == 4 ){
					*yoff = atoi(temp);
				}
				i = 0; k = 0; f++;
				continue;
			}
		}
		if( k ) temp[i++] = *p;
	}
	return 0;
}	
static int _check_off(char *buf)
{
	char *p;
	int f = 0;

	if( buf == NULL ) return 0;

	for( p = buf; *p ; p++ ){
		if( *p == '{' ){
			f = 1;
			continue;
		}
		if( f && *p == '}') return 1;
	}

	return 0;
}
int xtext_offt_data(offset_data_t **head,offset_data_t **tail,char *off_file)
{
	struct passwd *pwd;
	FILE *fp;

	char buf[8192],cstr[4],offset_file[128];
	unsigned int c0,c1;
	int xoff,yoff;

	*head = NULL; *tail = NULL;

	 pwd = getpwuid(getuid());
         sprintf(offset_file,"%s/%s/%s",pwd->pw_dir,CONF_DIR_NAME,off_file);

	 if((fp = fopen(offset_file,"r")) == NULL){
		 /*
		 fprintf(stderr,"Error: read offset_file xtext_f15.c\n");
		 */
		 *head = NULL; *tail = NULL;
		 return -1;
	 }
	 while(fgets(buf,8192,fp) != NULL){

		 if(buf[0] == '#' || buf[0] == '\n') continue;

		 if(_check_off(buf)){
			 _parse_data(buf,cstr,&c0,&c1,&xoff,&yoff);
			 _offt_data_insert(head,tail,cstr,c0,c1,xoff,yoff,0);
		 }

	 }
	 fclose(fp);

	 return 0;
}
static int xtext_clear_offset(IBUF_T *head)
{
	IBUF_T *ib;
	ITEXT_T *it;

	if(head == NULL) return -1;

	for(ib=head;ib!=NULL;ib=ib->next){
		for(it=ib->i_head;it!=NULL;it=it->next){
			it->off = 0;
		}
	}

	return 0;
}
int xtext_set_offset(IBUF_T *head,IBUF_T *tail,offset_t *ohead)
{
	IBUF_T *ib;
	ITEXT_T *it;
	offset_data_t *ot,*oh;

	if(head == NULL) return -1;
	if(ohead->main_off_head == NULL && ohead->sub_off_head == NULL) return -1;
	xtext_clear_offset(head);


	for(ib = head;ib != NULL;ib = ib->next){
		for(it = ib->i_head;it != NULL;it = it->next){

			oh = NULL;
			if( ChFlag(it->flag,_EJC) ){
				if( !ChFlag(it->flag,_SML) )
					oh = ohead->main_off_head;

			}
			if( ChFlag(it->flag,_SML) ){
				oh = ohead->sub_off_head;
			}

			if( oh != NULL ){
				for(ot = oh;ot != NULL;ot = ot->next){
					if((it->c1 == ot->c1) && (it->c2 == ot->c2)){
						SetOffset(&(it->off),ot->x_off,ot->y_off);
						break;
					}
				}
			}
		}
	}

	return 0;
}
